import { Editor } from '@tinymce/tinymce-react';
import React from 'react';
import PropTypes from 'prop-types';
import { message } from 'antd';

const MAX_LENGTH = 128 * 1024;
const MIN_LENGTH = 100 * 1024;
function EditorFormItem({
  contentStyle,
  getEditor,
  value = '',
  onChange,
  needMenu = false,
  height = '500px',
  disabled = false,
  initProps,
}) {
  let maxQ = 100;
  let minQ = 0;

  function handleChange(currentHtml) {
    const parser = new DOMParser();
    const doc = parser.parseFromString(currentHtml, 'text/html');
    const inputs = doc.querySelectorAll('input');
    const values = Array.from(inputs).map((input) => input.value);
    const textContent = doc.querySelector('p')?.textContent;
    const result = values.join('') + textContent;
    if (result.length > 2000) {
      message.error('输入长度不能超过2000个字符');
    } else {
      onChange(currentHtml);
    }
  }
  function compress(canvas, quality, callback) {
    const base64 = canvas.toDataURL('image/jpeg', quality / 100);
    if (base64.length <= MAX_LENGTH && base64.length >= MIN_LENGTH) {
      callback(base64);
      return;
    }
    if (base64.length > MAX_LENGTH) {
      // 压缩后文件大于最大值
      maxQ = quality;
    }
    if (base64.length < MIN_LENGTH) {
      // 压缩以后文件小于最小值
      minQ = quality;
    }
    compress(canvas, Math.ceil((minQ + maxQ) / 2), callback);
  }

  function imageUpload(file, callBack) {
    const reader = new FileReader();
    reader.readAsDataURL(file);
    reader.onload = function (e) {
      const newImage = new Image();
      newImage.src = e.target.result;
      newImage.setAttribute('crossOrigin', 'Anonymous'); // url为外域时需要
      newImage.onload = function () {
        // 使用压缩
        const canvas = document.createElement('canvas');
        const ctx = canvas.getContext('2d');
        canvas.width = this.width;
        canvas.height = this.height;
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        ctx.drawImage(this, 0, 0, canvas.width, canvas.height);

        const base64 = canvas.toDataURL('image/jpeg', 0.92);
        if (base64.length <= MAX_LENGTH) {
          callBack(base64);
        } else {
          maxQ = 100;
          minQ = 0;
          compress(canvas, 50, callBack);
        }
      };
    };
  }

  function handleInit(_, editor) {
    if (getEditor) {
      getEditor(editor);
    }
  }

  function handleClearFixedTag(ed) {
    const fixedTag = ed.dom.select('.highLight');
    if (fixedTag) {
      Object.keys(fixedTag).forEach(function (key) {
        fixedTag[key].classList.remove('highLight');
      });
    }
  }

  function handleFixedTag(ed, type, v) {
    handleClearFixedTag(ed);
    const domType = type.toString() === 'label' ? 'span' : type.toString();
    const tag = ed.dom.select(`${domType}[data-tag-id='${v}']`)[0];
    if (tag) {
      tag.scrollIntoView();
      tag.classList.add('highLight');
    } else {
      message.error('当前文档找不到此标签');
    }
  }

  return (
    <Editor
      tinymceScriptSrc={`${process.env.PUBLIC_URL}/tinymce/tinymce.min.js`}
      value={value}
      init={{
        height,
        menubar: needMenu,
        promotion: false,
        // statusbar: false,
        branding: false,
        plugins: [
          'advlist',
          'autolink',
          'link',
          'image',
          'lists',
          'charmap',
          'preview',
          'anchor',
          'print',
          'pagebreak',
          'searchreplace',
          'wordcount',
          'visualblocks',
          'code',
          'fullscreen',
          'insertdatetime',
          'media',
          'table',
          'paste',
          'code',
          'wordcount',
          'customtag',
          'inputtag',
          'deletetag',
          'specialtag',
          'complextag',
        ],
        toolbar: false,
        font_family_formats:
          "微软雅黑='微软雅黑';宋体='宋体';黑体='黑体';仿宋='仿宋';楷体='楷体';隶书='隶书';幼圆='幼圆';Andale Mono=andale mono,times;Arial=arial,helvetica,sans-serif;Arial Black=arial black,avant garde;Book Antiqua=book antiqua,palatino;Comic Sans MS=comic sans ms,sans-serif;Courier New=courier new,courier;Georgia=georgia,palatino;Helvetica=helvetica;Impact=impact,chicago;Symbol=symbol;Tahoma=tahoma,arial,helvetica,sans-serif;Terminal=terminal,monaco;Times New Roman=times new roman,times;Trebuchet MS=trebuchet ms,geneva;Verdana=verdana,geneva;Webdings=webdings;Wingdings=wingdings",
        font_size_formats: '8pt 10pt 12pt 14pt 16pt 18pt 24pt 36pt 48pt',
        language: 'zh_CN',
        language_url: `${process.env.PUBLIC_URL}/tinymce/langs/zh_CN.js`,
        file_picker_types: 'image',

        content_style:
          'body { font-family:仿宋; font-size:14pt; }' +
          `.highLight{background-color:yellow !important;} ${contentStyle}`,
        images_upload_handler: (blobInfo) => {
          return new Promise((resolve) => {
            const { type } = blobInfo.blob();
            if (blobInfo.blob().size > MAX_LENGTH) {
              imageUpload(blobInfo.blob(), resolve);
            } else {
              resolve(`data:${type};base64,${blobInfo.base64()}`);
            }
          });
        },
        file_picker_callback: (cb) => {
          const input = document.createElement('input');
          input.setAttribute('type', 'file');
          input.setAttribute('accept', 'image/*');

          input.onchange = function () {
            const file = this.files[0];
            imageUpload(file, cb);
          };
          input.click();
        },
        paste_data_images: true,
        paste_postprocess(plugin, args) {
          const tableNodes = args.node.getElementsByTagName('table');
          if (tableNodes.length > 0) {
            for (const i in tableNodes) {
              if (typeof tableNodes[i] === 'object') {
                tableNodes[i].setAttribute('border', 1);
              }
            }
          }
        },
        setup: function (ed) {
          ed.addCommand('CheckDeleteTag', function (type, v) {
            const domType = type.toString() === 'label' ? 'span' : type.toString();
            const tag = ed.dom.select(`${domType}[data-tag-id='${v}']`)[0];
            if (tag) {
              ed.editorManager.execCommand('DeleteTag', `${type},${v}`);
            }
          });
          ed.addCommand('ClearFixedTag', () => handleClearFixedTag(ed));
          ed.addCommand('GetInputTagValue', function (id, callBack) {
            const tagList = ed.dom.select(`input[data-tag-id='${id.toString()}']`);
            const tagValue = {};
            for (let i of tagList) {
              const tagId = i.getAttribute('id');
              tagValue[tagId] = i.value;
            }
            console.log('tagValue', tagValue);
            callBack(tagValue);
          });
          ed.addCommand('FixedTag', (type, v) => handleFixedTag(ed, type, v));
          ed.addCommand('AddSign', function (add, v) {
            const content = ed.dom.select('#tinymce')[0];
            v.forEach((i) => {
              const signDom = content.querySelector(`img[id='${i.id}']`);
              if (add && !signDom) {
                const img = document.createElement('img');
                img.id = i.id;
                img.style.position = 'absolute';
                img.style.left = `${i.xAxis}px`;
                img.style.top = `${i.yAxis}px`;
                img.style.outline = 'none';
                img.style.zIndex = '-1';
                img.src = i.signImgData;
                content.appendChild(img);
              } else {
                signDom.style.left = `${i.xAxis}px`;
                signDom.style.top = `${i.yAxis}px`;
              }
            });
          });
        },
        ...initProps,
      }}
      disabled={disabled}
      onInit={handleInit}
      onEditorChange={handleChange}
    />
  );
}

EditorFormItem.propTypes = {
  needMenu: PropTypes.bool,
  height: PropTypes.string,
  disabled: PropTypes.bool,
  contentStyle: PropTypes.string,
  value: PropTypes.string,
};

export default EditorFormItem;
