import React, { ReactNode, useEffect, useMemo, useState } from 'react';
import { Button, Col, Form, Row, Segmented, Select, Space } from 'antd';
import { DefaultOptionType } from 'rc-select/lib/Select';
import { DeleteOutlined, MinusCircleOutlined, PlusOutlined } from '@ant-design/icons';
import {
  comparisonOptions,
  conditionGroupDefaultValue,
  relationOption,
} from './SelectConditionConstant';
import styles from './select-condition.module.less';

interface SelectAssociationConditionProps {
  name: string | number;
  fieldOptions?: DefaultOptionType[];
  associationSubTableOptions?: DefaultOptionType[];
  onFieldSelect?: (fieldValue: string) => Promise<DefaultOptionType[]>;
}

interface ConditionGroupProps {
  name: string | number;
  baseNamePath?: (string | number)[];
  onRemoveGroup?: () => void;
  fieldOptions?: DefaultOptionType[];
  associationSubTableOptions?: DefaultOptionType[];
  onFieldSelect?: (fieldValue: string) => Promise<DefaultOptionType[]>;
  fieldComponentRender?: (
    fieldOptions: DefaultOptionType[] | undefined,
    onChange: () => any,
  ) => React.ReactNode;
  valueComponentRender?: () => ReactNode;
}
interface ConditionItemProps {
  name: number;
  baseNamePath: (string | number)[];
  onRemoveItem: () => void;

  disableRemove?: boolean;
  fieldOptions?: DefaultOptionType[];
  associationSubTableOptions?: DefaultOptionType[];
  onFieldSelect?: (fieldValue: string) => Promise<DefaultOptionType[]>;

  fieldComponentRender?: (
    fieldOptions: DefaultOptionType[] | undefined,
    onChange: () => any,
  ) => React.ReactNode;
  valueComponentRender?: () => ReactNode;
}

function ConditionItem(props: ConditionItemProps) {
  const {
    name,
    baseNamePath,
    onRemoveItem,
    disableRemove,
    fieldOptions,
    onFieldSelect,
    fieldComponentRender,
    valueComponentRender,
    associationSubTableOptions,
    ...restField
  } = props;
  const watchedNamePath = useMemo(() => baseNamePath.concat(name), [baseNamePath.toString(), name]);
  const itemValue = Form.useWatch(watchedNamePath);
  const form = Form.useFormInstance();
  const [options, setOptions] = useState<DefaultOptionType[]>([]);
  const [associationOptions, setAssociationOptions] = useState<DefaultOptionType[]>([]);

  useEffect(() => {
    if (itemValue && itemValue.relation == null) {
      if (itemValue.leftTableName) {
        onFieldSelect?.(itemValue.leftTableName).then((data) => {
          setOptions(data);
        });
      }
    }
  }, [itemValue?.leftTableName]);
  useEffect(() => {
    if (itemValue && itemValue.relation == null) {
      if (itemValue.rightTableName) {
        onFieldSelect?.(itemValue.rightTableName).then((associationData) =>
          setAssociationOptions(associationData),
        );
      }
    }
  }, [itemValue?.rightTableName]);

  function onFieldSelectChange() {
    form.setFieldValue(watchedNamePath.concat('leftTableField'), []);
  }
  function onFieldRightSelectChange() {
    form.setFieldValue(watchedNamePath.concat('rightTableField'), []);
  }

  if (itemValue && itemValue.relation !== undefined) {
    return (
      <ConditionGroup
        name={name}
        baseNamePath={baseNamePath}
        onRemoveGroup={onRemoveItem}
        fieldOptions={fieldOptions}
        onFieldSelect={onFieldSelect}
        fieldComponentRender={fieldComponentRender}
        valueComponentRender={valueComponentRender}
      />
    );
  }

  return (
    <div>
      <Space align="baseline">
        <Form.Item
          {...restField}
          name={[name, 'leftTableName']}
          rules={[
            {
              required: true,
              message: '请选择表名',
            },
          ]}
        >
          {fieldComponentRender ? (
            fieldComponentRender(fieldOptions, onFieldSelectChange)
          ) : (
            <Select
              style={{ minWidth: 150 }}
              placeholder="请选择表名"
              options={fieldOptions}
              onChange={onFieldSelectChange}
              showSearch
            />
          )}
        </Form.Item>
        <Form.Item
          {...restField}
          name={[name, 'leftTableField']}
          rules={[
            {
              required: true,
              message: '请选择字段',
            },
          ]}
        >
          {fieldComponentRender ? (
            fieldComponentRender(fieldOptions, onFieldSelectChange)
          ) : (
            <Select
              style={{ minWidth: 150 }}
              placeholder="请选择字段"
              optionFilterProp="label"
              options={options}
              showSearch
            />
          )}
        </Form.Item>
        <Form.Item
          {...restField}
          name={[name, 'joinOperator']}
          rules={[
            {
              required: true,
              message: '请选择关系符',
            },
          ]}
        >
          <Select style={{ width: 150 }} placeholder="请选择关系符" options={comparisonOptions} />
        </Form.Item>
        <Form.Item
          {...restField}
          name={[name, 'rightTableName']}
          rules={[
            {
              required: true,
              message: '请选择表名',
            },
          ]}
        >
          {fieldComponentRender ? (
            fieldComponentRender(fieldOptions, onFieldRightSelectChange)
          ) : (
            <Select
              style={{ minWidth: 150 }}
              placeholder="请选择表名"
              options={fieldOptions}
              onChange={onFieldRightSelectChange}
              showSearch
            />
          )}
        </Form.Item>
        <Form.Item
          {...restField}
          name={[name, 'rightTableField']}
          rules={[
            {
              required: true,
              message: '请选择字段',
            },
          ]}
        >
          {fieldComponentRender ? (
            fieldComponentRender(fieldOptions, onFieldRightSelectChange)
          ) : (
            <Select
              style={{ minWidth: 150 }}
              placeholder="请选择字段"
              optionFilterProp="label"
              options={associationOptions}
              showSearch
            />
          )}
        </Form.Item>
        {!disableRemove && (
          <MinusCircleOutlined
            className={styles.removeIcon}
            onClick={onRemoveItem}
            title="删除条件"
          />
        )}
      </Space>
    </div>
  );
}

function ButtonGroup(props: {
  onAddItem: () => void;
  onAddGroup: () => void;
  onRemoveGroup: (() => void) | undefined;
}) {
  return (
    <Space>
      <Button type="dashed" onClick={props.onAddItem} icon={<PlusOutlined />}>
        加条件
      </Button>
      <Button type="dashed" onClick={props.onAddGroup} icon={<PlusOutlined />}>
        加条件组
      </Button>
      {props.onRemoveGroup && (
        <Button type="dashed" onClick={props.onRemoveGroup} danger icon={<DeleteOutlined />}>
          删除条件组
        </Button>
      )}
    </Space>
  );
}

function ConditionGroup(props: ConditionGroupProps) {
  const {
    name,
    baseNamePath = [],
    onRemoveGroup,
    fieldOptions,
    onFieldSelect,
    fieldComponentRender,
    valueComponentRender,
    associationSubTableOptions,
    ...restField
  } = props;

  const namePathList = [name, 'children'];
  return (
    <Row className={styles.conditionGroup}>
      <Col className={styles.relation}>
        <Form.Item
          noStyle
          name={[name, 'relation']}
          initialValue={conditionGroupDefaultValue.relation}
        >
          <Segmented options={relationOption} />
        </Form.Item>
      </Col>
      <Col>
        <Form.Item noStyle shouldUpdate>
          {() => (
            <Form.List name={namePathList} initialValue={conditionGroupDefaultValue.children}>
              {(fields, { add, remove }) => {
                const conditionNamePath = baseNamePath.concat(namePathList);
                return (
                  <div className={styles.conditionItems}>
                    {fields.map(({ name: fieldName, key }) => (
                      <ConditionItem
                        key={key}
                        fieldOptions={fieldOptions}
                        associationSubTableOptions={associationSubTableOptions}
                        disableRemove={fields.length === 1}
                        name={fieldName}
                        baseNamePath={conditionNamePath}
                        onRemoveItem={() => remove(fieldName)}
                        onFieldSelect={onFieldSelect}
                        fieldComponentRender={fieldComponentRender}
                        valueComponentRender={valueComponentRender}
                      />
                    ))}
                    <ButtonGroup
                      onAddItem={() => add()}
                      onAddGroup={() => add(conditionGroupDefaultValue)}
                      onRemoveGroup={onRemoveGroup}
                    />
                  </div>
                );
              }}
            </Form.List>
          )}
        </Form.Item>
      </Col>
    </Row>
  );
}

function SelectAssociationCondition(props: SelectAssociationConditionProps) {
  return <ConditionGroup {...props} />;
}

SelectAssociationCondition.defaultProps = {};

export default SelectAssociationCondition;
