import React from 'react';
import PropTypes from 'prop-types';
import { Col, Form, Input, Row } from 'antd';
import pick from 'lodash/pick';

export function RenderFormItem(props) {
  const { formItem } = props;
  const {
    key,
    name,
    label,
    component: InputComponent = Input,
    shouldUpdate,
    shouldRender,
  } = formItem;

  const formItemProps = {
    ...formItem.formItemLayout,
    ...pick(formItem, [
      'colon',
      'dependencies',
      'extra',
      'getValueFromEvent',
      'getValueProps',
      'hasFeedBack',
      'help',
      'hidden',
      'htmlFor',
      'initialValue',
      'noStyle',
      'label',
      'labelAlign',
      'labelCol',
      'name',
      'normalize',
      'tooltip',
      'trigger',
      'validateFirst',
      'validateTrigger',
      'valuePropName',
      'wrapperCol',
    ]),
    ...formItem.formItemProps,
  };

  const rules = Array.isArray(formItem.rules) ? formItem.rules.slice() : [];
  if (formItem.required) {
    // Handle rules
    rules.push({ required: true, message: `请输入 ${label || name || key}` });
  }

  const componentProps = {
    ...props.componentProps,
    ...formItem.componentProps,
  };

  componentProps.disabled = componentProps.disabled || formItem.disabled || props.disabled;

  const item = (
    <Form.Item name={name || key} {...formItemProps} rules={rules}>
      <InputComponent {...componentProps}>{formItem.children}</InputComponent>
    </Form.Item>
  );

  if (shouldUpdate && shouldRender) {
    return (
      <Form.Item shouldUpdate={shouldUpdate} noStyle>
        {(form) => shouldRender(form) && item}
      </Form.Item>
    );
  }

  return item;
}

function FormBuilder(props) {
  function renderLayout(formItem, index) {
    const { span, visible, colLayout, ...restFormItem } = formItem;
    const colProps = span ? { span } : { ...props.colLayout, ...colLayout };

    if (visible === false) {
      if (colProps.style) {
        colProps.style.display = 'none';
      } else {
        colProps.style = { display: 'none' };
      }
    }

    return (
      <Col key={formItem.key || formItem.name || index} {...colProps}>
        <RenderFormItem formItem={restFormItem} {...props} />
      </Col>
    );
  }

  return <Row gutter={props.gutter}>{props.items.map(renderLayout)}</Row>;
}

FormBuilder.propTypes = {
  gutter: PropTypes.number,
  colLayout: PropTypes.object,
  componentProps: PropTypes.object,
  disabled: PropTypes.bool,
  items: PropTypes.arrayOf(
    PropTypes.shape({
      key: PropTypes.string,
      name: PropTypes.string.isRequired,
      label: PropTypes.string,
      tooltip: PropTypes.string,
      required: PropTypes.bool,
      disabled: PropTypes.bool,
      visible: PropTypes.bool,
      component: PropTypes.elementType,
      componentProps: PropTypes.object,
      span: PropTypes.number,
      colLayout: PropTypes.object,
      formItemLayout: PropTypes.object,
      formItemProps: PropTypes.object,
      shouldRender: PropTypes.func,
    }),
  ).isRequired,
};

FormBuilder.defaultProps = {
  gutter: 0,
  colLayout: { span: 24 },
};

export default FormBuilder;
