import React, { ReactNode } from 'react';
import { Alert, Card, Col, Form, Input, Row, Table } from 'antd';
import classNames from 'classnames';
import ToolBar from '../../ToolBar';
import MenuSteps from './MenuSteps';
import AdvSearch from './AdvSearch';
import './IndexPage.less';
import { useIndexPageContext } from './index';

interface IndexPageProps {
  pageTitle: string;
  showMenu?: boolean;
  searchBarItemList?: any[];
  topExtra?: ReactNode;
  simpleSearchForm?: ReactNode;
  tableColumns: any[];
  pageExtra?: ReactNode;
  tabList?: any[];
  onTabChange?: (key: string) => void;
  tableProps?: object;
  toolBarItemList?: any[];
  alertInfo?: object;
  alertType?: 'success' | 'info' | 'warning' | 'error';
  alertMessage?: ReactNode;
  alertDescription?: ReactNode;
}

function IndexPage(props: IndexPageProps) {
  const {
    pageTitle,
    pageExtra,
    showMenu = true,
    tabList,
    onTabChange,
    searchBarItemList,
    topExtra,
    simpleSearchForm,
    tableColumns,
    tableProps,
    toolBarItemList,
    alertInfo,
    alertType = 'info',
    alertMessage,
    alertDescription,
  } = props;

  const {
    tableData,
    tableLoading,
    pagination,
    menuList,
    fetchTableData,
    selectedRowKeys,
    setSelectedRowKeys,
    searchForm,
  } = useIndexPageContext();
  const hasToolBarItemList: boolean = Array.isArray(toolBarItemList) && toolBarItemList.length > 0;
  const rowSelection = hasToolBarItemList
    ? { selectedRowKeys, onChange: setSelectedRowKeys }
    : undefined;

  function handleTableChange(page: any, filters: any, sorter: any) {
    const param = {
      current: page.current,
      pageSize: page.pageSize,
      sortField: sorter.field,
      sortOrder: sorter.order,
      ...filters,
    };
    fetchTableData(param);
  }

  return (
    <div className="index-page">
      {showMenu && (
        <Card className="menu-card">
          <MenuSteps menu={menuList} />
        </Card>
      )}
      <Card
        className={showMenu ? 'content-card-with-menu' : 'content-card'}
        title={pageTitle && !showMenu && pageTitle}
        extra={!showMenu && pageExtra}
        tabList={tabList}
        onTabChange={onTabChange}
      >
        {alertMessage && (
          <Alert
            style={{ marginBottom: 24 }}
            message={alertMessage}
            description={alertDescription}
            type={alertType}
            showIcon
            {...alertInfo}
          />
        )}
        {topExtra || (
          <Row className="content-table-header">
            <Col span={12}>{showMenu && pageExtra}</Col>
            <Col span={12}>
              <Form layout="inline" form={searchForm} className="search-bar">
                <Form.Item name="keyword" style={{ marginRight: 0, width: 200 }}>
                  <Input.Search
                    placeholder="模糊查询"
                    allowClear
                    onSearch={(value) => {
                      searchForm.setFieldsValue({
                        keyword: value || undefined,
                      });
                      fetchTableData();
                    }}
                  />
                </Form.Item>
                {simpleSearchForm}
              </Form>
            </Col>
          </Row>
        )}
        {searchBarItemList && <AdvSearch itemList={searchBarItemList} />}

        <div className="table-alert" style={{ display: hasToolBarItemList ? 'block' : 'none' }}>
          <Alert
            message={
              <span>
                已选择&nbsp;
                <a style={{ fontWeight: 600 }}>{selectedRowKeys.length}</a>
                &nbsp;项
                {selectedRowKeys.length > 0 && (
                  <a onClick={() => setSelectedRowKeys()} style={{ marginLeft: 24 }}>
                    取消已选
                  </a>
                )}
              </span>
            }
            type="info"
            showIcon
          />
        </div>

        <Table
          rowKey="id"
          size="middle"
          bordered
          columns={tableColumns}
          dataSource={tableData}
          loading={tableLoading}
          rowSelection={rowSelection}
          onChange={handleTableChange}
          pagination={{
            ...pagination,
            showQuickJumper: true,
            showSizeChanger: true,
            showTotal: (total) => `共 ${total} 项`,
          }}
          {...tableProps}
        />
        {hasToolBarItemList && (
          <ToolBar
            className={classNames('toolbar', {
              'toolbar-shift': tableData.length > 0,
            })}
            itemList={toolBarItemList || []}
          />
        )}
      </Card>
    </div>
  );
}

export default IndexPage;
