import React, { useEffect, useState } from 'react';
import { Form } from 'antd';
import { useRequest } from 'ahooks';
import request from '../../../utils/request';

export const IndexPageContext = React.createContext({} as any);

type indexPageOption = {
  fetchUrl: string;
  fetchParam?: object;
  fetchMenu?: boolean;
};

export interface MenuItem {
  name: string;
  icon: string;
  link: string;
  dataCount: number;
  permission: boolean;
}

function withIndexPage({ fetchUrl, fetchParam, fetchMenu = true }: indexPageOption) {
  return (WrappedComponent: any) =>
    function IndexPageContainer(props: any) {
      const [currentFetchUrl, setCurrentFetchUrl] = useState(fetchUrl);
      const [currentFetchParam, setCurrentFetchParam] = useState(fetchParam);
      const [selectedRows, setSelectedRows] = useState([]);
      const [selectedRowKeys, setSelectedRowKeys] = useState([]);
      const [tableData, setTableData] = useState([]);
      const [pagination, setPagination] = useState({
        total: 0,
        current: 1,
        pageSize: 10,
      });
      const [menuList, setMenuList] = useState<MenuItem[]>([]);
      const [menuFetched, setMenuFetched] = useState(false);
      const [stepKeyword, setStepKeyword] = useState();
      const [searchForm] = Form.useForm();

      const {
        loading: tableLoading,
        run,
        refresh,
      } = useRequest(fetchTableData, {
        manual: true,
      });

      const { loading, run: menuRun } = useRequest(fetchMenuData, {
        manual: true,
      });

      useEffect(() => {
        if (fetchMenu) {
          menuRun();
        }
      }, []);

      useEffect(() => {
        if (fetchMenu) {
          if (menuFetched) {
            run();
          }
        } else {
          run();
        }
      }, [currentFetchUrl, currentFetchParam, menuFetched]);

      function setSelectedRowAndKeys(keys = [], rows = []) {
        setSelectedRowKeys(keys);
        setSelectedRows(rows);
      }

      function fetchMenuData() {
        return request.getWMessage('system-menu-m-001', { url: props.match.path }).then((res) => {
          if (Array.isArray(res.data)) {
            const menu = res.data.map((item: any) => ({
              name: item.menuName,
              icon: item.menuIco,
              link: item.menuUrl,
              dataCount: item.dataCount,
              permission: item.permission,
            }));
            setMenuList(menu);
            // getStepKeyword
            const currentMenu = res.data.find((item: any) => item.menuUrl === props.match.path);

            if (currentMenu && currentMenu.stepKeyword) {
              setStepKeyword(currentMenu.stepKeyword);
            }
            setMenuFetched(true);
          }
        });
      }

      // 用传入的url和param请求
      function fetchTableData(params?: any) {
        if (!fetchUrl) {
          console.error('withIndexPage: need to pass in fetchRequestUrl');
          return Promise.reject();
        }
        return request
          .getWMessage(currentFetchUrl, {
            stepKeyword,
            ...currentFetchParam,
            ...searchForm.getFieldsValue(),
            pageSize: pagination.pageSize,
            ...params,
          })
          .then((res: any) => {
            setTableData(res.data.list);
            setPagination({
              current: res.data.current,
              total: res.data.total,
              pageSize: res.data.pageSize,
            });
            setSelectedRowAndKeys();
          });
      }

      return (
        <IndexPageContext.Provider
          value={{
            selectedRows,
            selectedRowKeys,
            tableData,
            tableLoading: loading || tableLoading,
            pagination,
            menuList,
            setSelectedRowKeys: setSelectedRowAndKeys,
            fetchTableData,
            refreshTableData: refresh,
            searchForm,
          }}
        >
          <WrappedComponent
            selectedRows={selectedRows}
            selectedRowKeys={selectedRowKeys}
            setSelectedRowKeys={setSelectedRowAndKeys}
            tableData={tableData}
            pagination={pagination}
            setTableData={setTableData}
            fetchTableData={fetchTableData}
            fetchMenuData={fetchMenuData}
            setFetchUrl={setCurrentFetchUrl}
            setFetchParam={setCurrentFetchParam}
            searchForm={searchForm}
            {...props}
          />
        </IndexPageContext.Provider>
      );
    };
}

export default withIndexPage;
