import { Button, Drawer, Form, Modal, Select, Space, Table, Typography } from 'antd';
import SearchBar from '@/components/forms/SearchBar';
import React, { useState } from 'react';
import { syncOption, TaskStatus } from '@/pages/system/syncProcess/ProcessStatus';
import RangePickerString from '@/components/forms/RangePickerString';
import { SynchronizationDetailsCol } from '@/components/system/LogDetailsInfoCol';
import { useBoolean, useRequest } from 'ahooks';
import request from '@/utils/request';
import { ExclamationCircleFilled } from '@ant-design/icons';

export default function SynchronizationDetailsDrawer(props) {
  const { id: businessId, visible, setFalse, syncRecordsId } = props;
  const [content, setContent] = useState();
  const [searchForm] = Form.useForm();
  const [pagination, setPagination] = useState({ total: 0, current: 1, pageSize: 10 });
  const [open, { setTrue: setOpenTrue, setFalse: setOpenFalse }] = useBoolean(false);
  const searchBarItem = [
    {
      name: 'keyword',
      key: 'keyword',
      label: '模糊搜索',
      componentProps: { placeholder: '主机名/业务主键', width: 200 },
    },
    {
      name: 'operateType',
      key: 'operateType',
      label: '操作类型',
      component: Select,
      componentProps: {
        style: { minWidth: 200 },
        options: syncOption,
      },
    },
    {
      key: 'dateRangeArray',
      label: '时间范围',
      component: RangePickerString,
      componentProps: {
        allowClear: true,
        allowEmpty: [true, true],
      },
    },
  ];
  const synchronizationStatus = [
    {
      title: '同步状态',
      width: 80,
      align: 'center',
      dataIndex: 'syncResult',
      key: 'syncResult',
      render: (text, record) => {
        if (text === TaskStatus.SUCCESS) {
          return <span className="text-green-6">成功</span>;
        }
        if (text === TaskStatus.ERROR) {
          return (
            <Space>
              <span className="text-red-6">失败</span>
              <ExclamationCircleFilled
                className="text-orange-6 text-xl"
                onClick={() => handleClick(record.errorCause)}
              />
            </Space>
          );
        }
        return <span className="text-gray-6">未发送</span>;
      },
    },
  ];
  function handleClick(errorContent) {
    setOpenTrue();
    setContent(errorContent);
  }
  const { data = { list: [] }, run } = useRequest(
    async (params) => {
      const res = await fetchTableData(params);
      return res.data;
    },
    {
      refreshDeps: [visible],
      manual: false,
    },
  );
  function fetchTableData(params) {
    return request
      .getWMessage('/syndata/syncTaskLogDetails/getTargetTenantDetails', {
        businessId,
        syncRecordsId,
        ...params,
      })
      .then((res) => {
        setPagination({
          current: res.data.current,
          total: res.data.total,
          pageSize: res.data.pageSize,
        });
        return res;
      });
  }
  function onChange(page, filters) {
    const params = {
      current: page.current,
      pageSize: page.pageSize,
      ...filters,
    };
    run({ ...params, ...searchForm.getFieldsValue() });
  }
  function handleClose() {
    searchForm.resetFields();
    setOpenFalse();
  }
  return (
    <>
      <Drawer
        title="同步详情"
        open={visible}
        width={'65vw'}
        maskClosable
        closeIcon
        onClose={setFalse}
        footer={
          <div className="text-right">
            <Button onClick={setFalse}>关闭</Button>
          </div>
        }
      >
        <SearchBar
          onSearch={(value) => {
            run({ ...value });
          }}
          form={searchForm}
          itemList={searchBarItem}
        />
        <Table
          columns={SynchronizationDetailsCol.concat(synchronizationStatus)}
          dataSource={data.list}
          rowKey="id"
          onChange={onChange}
          pagination={{
            ...pagination,
            showQuickJumper: true,
            showSizeChanger: false,
            showTotal: (total) => `共 ${total} 项`,
          }}
          bordered
        />
      </Drawer>
      <Modal
        closeIcon
        title="失败明细"
        open={open}
        centered
        onCancel={handleClose}
        width={700}
        footer={<Button onClick={handleClose}>关闭</Button>}
      >
        <Typography>
          <Typography.Paragraph>{content}</Typography.Paragraph>
        </Typography>
      </Modal>
    </>
  );
}
