import React, { CSSProperties, useEffect, useState } from 'react';
import { PlusOutlined } from '@ant-design/icons';
import { Button, Form, Input, message, Pagination, Popconfirm, Space, Table } from 'antd';
import { TableProps } from 'antd/es/table/InternalTable';
import { Rule } from 'rc-field-form/lib/interface';
import { ColumnsType } from 'antd/lib/table';
// @ts-ignore
import { v4 as uuid } from 'uuid';

type editableFunction = (value: any, record: any, index: number) => boolean;
type renderFormItemFunction = (value: any, record: any, index: number) => React.ReactNode;
type EditColumnsType = ColumnsType<any> & {
  editable?: boolean | editableFunction;
  renderFormItem?: React.ReactNode | renderFormItemFunction;
  componentProps?: object;
  formItemProps?: object;
  required?: boolean;
  rules?: Rule[];
};
export interface EditProTableProps extends Omit<TableProps<any>, 'onChange' | 'onRow' | 'columns'> {
  rowKey?: string;
  className?: string;
  style?: CSSProperties;
  disabled?: boolean;
  value?: any[];
  onChange?: (tableData: any[]) => void;
  onRow?: (
    data: any,
    edit: Function,
    index?: number,
  ) => React.HTMLAttributes<any> | React.TdHTMLAttributes<any>;
  onTableChange?: (page: number, pageSize: number) => void;
  creatorButtonText?: boolean | React.ReactNode;
  isMultiple?: boolean;
  columnsOptions?: (
    record: object,
    index: number,
    onSave: Function,
    onDelete: Function,
  ) => React.ReactNode;
  handleSave?: (record: object, tableData: any[]) => Promise<any>;
  handleDelete?: (key: any, record: object, tableData: any[]) => Promise<any>;
  optionColWidth?: string | number;
  columns?: EditColumnsType;
}

interface EditableCellProps {
  editing: boolean;
  dataIndex?: string;
  title?: React.ReactNode;
  children?: React.ReactNode;
  component?: string;
  componentProps?: object;
  required?: boolean;
  rulesProp?: Rule[];
  formItemProps?: object;
  renderFormItem?: React.ReactNode;
  rowId: any;
}

function EditableCell(props: EditableCellProps) {
  const {
    editing,
    dataIndex,
    title,
    children,
    component = Input,
    componentProps,
    required = false,
    rulesProp = [],
    formItemProps = {},
    renderFormItem,
    rowId,
    ...restProps
  } = props;
  const EditComponent = component;
  let rules = rulesProp;
  if (required) {
    rules = rulesProp.concat({ required: true, message: `请填写${title}` });
  }

  function getInput() {
    return (
      renderFormItem ?? (
        // @ts-ignore
        <Form.Item name={[rowId, dataIndex]} className="mb-0" rules={rules} {...formItemProps}>
          <EditComponent {...componentProps} />
        </Form.Item>
      )
    );
  }

  return (
    <td {...restProps}>
      {editing ? getInput() : <div style={{ wordBreak: 'break-word' }}>{children}</div>}
    </td>
  );
}

function EditProTable(props: EditProTableProps) {
  const {
    rowKey = 'id',
    columns = [],
    style,
    disabled,
    className,
    value = [],
    onChange = () => null,
    onTableChange,
    creatorButtonText = '添加',
    isMultiple = false,
    columnsOptions,
    handleSave,
    handleDelete,
    pagination = false,
    onRow,
    optionColWidth = 140,
    ...restProps
  } = props;
  const [editingKey, setEditingKey] = useState<any[]>([]);
  const [form] = Form.useForm();
  const [tableData, setTableData] = useState<any[]>([]);

  useEffect(() => {
    setTableData(value);
  }, [value]);

  function editRow(record: object) {
    if (editingKey.length > 0 && !isMultiple) {
      message.error('请先保存正在填写的信息!');
    } else {
      // @ts-ignore
      form.setFieldsValue({ [record[rowKey]]: record });
      // @ts-ignore
      setEditingKey(editingKey.concat(record[rowKey]));
    }
  }

  function handleCancel(record: any) {
    form.resetFields([record[rowKey]]);
    setEditingKey(editingKey.filter((i) => i !== record[rowKey]));
    if (record.catchAdd) {
      setTableData(tableData.filter((i) => record[rowKey] !== i[rowKey]));
    }
  }

  // @ts-ignore
  const isEditing = (record: object) => editingKey.includes(record[rowKey]);

  function handleOk(record: any) {
    const filedMap = columns.map((c: any) => [record[rowKey], c.dataIndex || '']);
    form.validateFields(filedMap).then((values) => {
      const newList = tableData.slice();
      const index = newList.findIndex((item) => record[rowKey] === item[rowKey]);
      const rowNew = { ...record, ...values[record[rowKey]] };
      if (rowNew.catchAdd) {
        delete rowNew[rowKey];
      }
      newList[index] = rowNew;

      if (handleSave) {
        handleSave(rowNew, newList).then(() => {
          setEditingKey(editingKey.filter((i) => i !== record[rowKey]));
          onChange(newList);
        });
      } else {
        setEditingKey(editingKey.filter((i) => i !== record[rowKey]));
        setTableData(newList);
        onChange(newList);
      }
    });
  }

  function addRow() {
    if (editingKey.length > 0 && !isMultiple) {
      message.error('请先保存正在填写的信息!');
    } else {
      const rowKeyValue = uuid().replace(/-/g, ''); // @ts-ignore;
      setTableData([
        ...tableData,
        {
          [rowKey]: rowKeyValue,
          catchAdd: true,
        },
      ]);
      form.resetFields([rowKeyValue]);
      setEditingKey([...editingKey, rowKeyValue]);
    }
  }

  function delRow(key: string, record: object) {
    const newTable = tableData.filter((i) => key !== i[rowKey]);
    if (handleDelete) {
      handleDelete(key, record, newTable).then(() => {
        setTableData(newTable);
        onChange(newTable);
      });
    } else {
      setTableData(newTable);
      onChange(newTable);
    }
  }

  const optionRender = (record: any, index: any) => {
    if (columnsOptions) {
      return columnsOptions(record, index, editRow, delRow);
    }
    return (
      <>
        <Button
          size="small"
          type="link"
          disabled={editingKey.length > 0 && !isMultiple}
          onClick={() => editRow(record)}
        >
          编辑
        </Button>
        <Popconfirm
          title="确认删除吗?"
          onConfirm={() => delRow(record[rowKey], record)}
          disabled={editingKey.length > 0 && !isMultiple}
        >
          <Button size="small" type="link" disabled={editingKey.length > 0 && !isMultiple}>
            删除
          </Button>
        </Popconfirm>
      </>
    );
  };

  const columnsUse = disabled
    ? columns
    : [
        ...columns,
        {
          title: '操作',
          align: 'center',
          width: optionColWidth,
          editable: false,
          render: (text: any, record: any, index: any) => {
            const editable = isEditing(record);
            return (
              <Space>
                {editable ? (
                  <>
                    <Button size="small" type="link" onClick={() => handleOk(record)}>
                      保存
                    </Button>
                    <Button size="small" type="link" onClick={() => handleCancel(record)}>
                      取消
                    </Button>
                  </>
                ) : (
                  optionRender(record, index)
                )}
              </Space>
            );
          },
        },
      ];

  const columnsNew = columnsUse.map(({ editable = true, renderFormItem, ...col }: any) => ({
    ...col,
    onCell: (record: object, rowIndex: number) => {
      // @ts-ignore
      const val = record[col.dataIndex];
      let editableNew = editable;
      if (editable instanceof Function) {
        editableNew = editable(val, record, rowIndex);
      }
      if (!editableNew) {
        return col;
      }
      let renderFormItemNew = renderFormItem;
      if (renderFormItem instanceof Function) {
        renderFormItemNew = renderFormItem(val, record, rowIndex);
      }

      return {
        dataIndex: col.dataIndex,
        title: col.title,
        editing: editableNew && isEditing(record),
        component: col.component,
        componentProps: col.componentProps,
        required: col.required,
        rulesProp: col.rules,
        renderFormItem: renderFormItemNew,
        formItemProps: col.formItemProps,
        // @ts-ignore
        rowId: record[rowKey],
      };
    },
  }));
  return (
    <div className={className} style={style}>
      <Form form={form} component={false}>
        <Table
          showHeader
          bordered
          columns={columnsNew}
          components={{
            body: {
              cell: EditableCell,
            },
          }}
          rowClassName="editableProRow"
          dataSource={tableData}
          rowKey={rowKey}
          pagination={false}
          onRow={onRow ? (data: any, index?: number) => onRow(data, editRow, index) : undefined}
          {...restProps}
        />
      </Form>
      {!disabled && creatorButtonText && (
        <Button type="dashed" className="mt-[12px]" block icon={<PlusOutlined />} onClick={addRow}>
          {creatorButtonText}
        </Button>
      )}
      {pagination && (
        <Pagination
          className="text-right my-[16px]"
          pageSize={10}
          {...pagination}
          onChange={onTableChange}
        />
      )}
    </div>
  );
}

EditProTable.propTypes = {};

EditProTable.defaultProps = {};

export default EditProTable;
