import { useRef } from 'react';
import { message } from 'antd';

enum CAIssueType {
  gxca,
}

interface CAInfo {
  startTime: string;
  endTime: string;
  caNumber: string;
  caTitle: string;
  caType: string;
  officeCode: string;
}

function useCAInfo() {
  const iframeRef = useRef({ contentWindow: window as any });
  const caTypeRef = useRef(CAIssueType.gxca);

  function getSN(): PromiseLike<string> {
    const win = iframeRef.current.contentWindow;
    if (caTypeRef.current === CAIssueType.gxca && win.gxcaPKI) {
      const { code, certinfo, errMsg } = win.gxcaPKI.getCertInfo(7);
      if (code === '0') {
        return Promise.resolve(certinfo);
      }
      if (errMsg && errMsg.toString() === '{}') {
        message.error('无法读取CA插件, 请尝试使用CA助手修复或重装插件');
        console.error('CA error:', code, errMsg);
      } else {
        message.error(errMsg || '未知CA插件错误, 请联系管理员');
        console.error('CA error:', code, errMsg);
      }
      return Promise.reject({ code: -1, msg: errMsg });
    }

    return Promise.reject({ code: -1, msg: '尚未支持的CA类型' });
  }

  function getAllInfo(): PromiseLike<CAInfo> {
    const win = iframeRef.current.contentWindow;
    if (caTypeRef.current === CAIssueType.gxca && win.gxcaPKI) {
      const { code, errMsg, ...certInfo } = win.gxcaPKI.getAllCertInfo();
      if (code === '0') {
        return Promise.resolve({
          startTime: certInfo.notAfter,
          endTime: certInfo.notBefore,
          caNumber: certInfo.serial,
          caTitle: certInfo.subject,
          caType: certInfo.issuer,
          officeCode: certInfo.postalcode,
        });
      }

      if (errMsg && errMsg.toString() === '{}') {
        message.error('无法读取CA插件, 请尝试使用CA助手修复或重装插件');
        console.error('CA error:', code, errMsg);
      } else {
        message.error(errMsg || '未知CA插件错误, 请联系管理员');
        console.error('CA error:', code, errMsg);
      }
      return Promise.reject({ code: -1, msg: errMsg });
    }

    return Promise.reject({ code: -1, msg: '尚未支持的CA类型' });
  }

  return {
    iframeRef,
    caTypeRef,
    getSN,
    getAllInfo,
  };
}

export default useCAInfo;
