import React, { useEffect, useState } from 'react';
import {
  Alert,
  Button,
  Card,
  Col,
  Drawer,
  Form,
  Modal,
  Popconfirm,
  Radio,
  Row,
  Space,
  Table,
  Tag,
} from 'antd';
import { v4 as uuid } from 'uuid';
import { updateInfo } from '@/pages/system/tenant/TenentColumns';
import {
  CharacteristicsTypeName,
  HostType,
  HostTypeName,
  hostTypeOption,
  TenantKey,
} from '@/const/TenantType';
import FormBuilder from '@/components/forms/FormBuilder';
import { useBoolean, useRequest } from 'ahooks';
import request from '@/utils/request';
import styles from '@/components/hoc/IndexPage/IndexPageNew.module.less';
import { EditableProTable } from '@ant-design/pro-table';
import SearchBar from '@/components/forms/SearchBar';

function HostGroupManagement() {
  const [visible, { setTrue, setFalse }] = useBoolean(false);
  const [
    characteristicVisible,
    { setTrue: setCharacteristicTrue, setFalse: setCharacteristicFalse },
  ] = useBoolean(false);
  const [dataSource, setDataSource] = useState([]);
  const [tabList, setTabList] = useState();
  const [tenantKeyMap, setTenantKeyMap] = useState({});
  const [editTableKeys, setEditTableKeys] = useState([]);
  const [tenantryHostId, setTenantryHostId] = useState();
  const [currentKey, setCurrentKey] = useState();
  const [featureType, setFeatureType] = useState(TenantKey.PURCHASER);
  const [form] = Form.useForm();
  const [searchForm] = Form.useForm();
  const [pagination, setPagination] = useState({ total: 0, current: 1, pageSize: 10 });
  const formItemLayout = { labelCol: { span: 4 }, wrapperCol: { span: 18 } };

  const searchBarItem = [
    {
      key: 'keyword',
      label: '模糊搜索',
      componentProps: { placeholder: '主机名称/标识', style: { width: 230 } },
    },
    {
      key: 'usageType',
      label: '用途类型',
      component: Radio.Group,
      componentProps: {
        options: [
          {
            label: '全部',
            value: '',
          },
          { value: HostType.BUSINESS, label: '业务库' },
          { value: HostType.QUERY, label: '查询库' },
        ],
      },
    },
  ];
  const tableCol = [
    { title: '序号', align: 'center', width: 80, render: (text, record, index) => index + 1 },
    {
      title: '主机名称',
      dataIndex: 'hostName',
      key: 'hostName',
      width: 180,
      align: 'center',
    },
    {
      title: '用途类型',
      dataIndex: 'usageType',
      key: 'usageType',
      align: 'center',
      width: 180,
      render: (text) => (
        <Tag color={text === HostType.BUSINESS ? 'orange' : 'blue'} className="py-1 px-4">
          {HostTypeName[text]}
        </Tag>
      ),
    },
    {
      title: '主机标识',
      width: 180,
      align: 'center',
      dataIndex: 'hostIdentification',
      key: 'hostIdentification',
    },
    ...updateInfo,
    {
      title: '操作',
      key: 'operation',
      align: 'center',
      width: 120,
      render: (text, record) => {
        return (
          <Space>
            <Button type="link" size="small" onClick={() => handleEdit(record)}>
              编辑
            </Button>
            <Popconfirm title="您确定删除吗?" onConfirm={() => handleDelete(record.id)}>
              <Button type="link" size="small">
                删除
              </Button>
            </Popconfirm>
            <Button
              type="link"
              size="small"
              onClick={() => handleEditCharacteristic(record.id, record.featureType)}
            >
              特征值
            </Button>
          </Space>
        );
      },
    },
  ];
  const formItems = [
    { name: 'id', key: 'id', visible: false },
    { name: 'tenantryId', key: 'tenantryId', visible: false },
    {
      name: 'tenantryName',
      key: 'tenantryName',
      label: ' 主机群',
      disabled: true,
      required: true,
    },
    {
      name: 'hostName',
      key: 'hostName',
      label: '主机名称',
      required: true,
    },
    {
      key: 'usageType',
      name: 'usageType',
      label: '用途类型',
      required: true,
      initialValue: HostType.BUSINESS,
      component: Radio.Group,
      componentProps: {
        options: hostTypeOption,
      },
    },
    {
      name: 'hostIdentification',
      key: 'hostIdentification',
      label: '主机标识',
      required: true,
    },
    {
      name: 'topicName',
      key: 'topicName',
      label: '主机主题',
      required: true,
    },
    {
      name: 'dbName',
      key: 'dbName',
      label: '库名',
      required: true,
    },
    {
      name: 'serviceName',
      key: 'serviceName',
      label: '服务名称',
      required: true,
    },
  ];
  const characteristicColumns = [
    {
      title: '序号',
      dataIndex: 'id',
      align: 'center',
      editable: false,
      width: 80,
      render: (text, record, index) => index + 1,
    },
    { title: '特征值', align: 'center', width: 180, dataIndex: 'featureKey', key: 'featureKey' },
    { title: '说明', width: 180, dataIndex: 'featureExplain', key: 'featureExplain' },
    {
      title: '操作',
      align: 'center',
      width: 180,
      dataIndex: 'option',
      valueType: 'option',
      render: (text, record, _, action) => {
        return (
          <Space>
            <Button
              type="link"
              size="small"
              onClick={() => {
                action?.startEditable?.(record.id);
              }}
            >
              编辑
            </Button>
            <Popconfirm title="确认删除吗?" onConfirm={() => handleDeleteFeature(record.id)}>
              <Button type="link" size="small">
                删除
              </Button>
            </Popconfirm>
          </Space>
        );
      },
    },
  ];
  function fetchTableData(params) {
    return request.getWMessage('/syndata/tenantryHost/getTenantryHost', params).then((res) => {
      setPagination({
        current: res.data.current,
        total: res.data.total,
        pageSize: res.data.pageSize,
      });
      return res;
    });
  }
  const {
    data = { list: [] },
    run,
    refresh,
  } = useRequest(
    async (params) => {
      const res = await fetchTableData(params);
      const { list, total } = res.data || {};
      return { list, total };
    },
    {
      manual: false,
    },
  );
  useEffect(() => {
    fetchTabList();
  }, []);
  function handleEdit(record) {
    form.setFieldsValue(record);
    setTrue();
  }
  function getCharacteristic(id) {
    return request
      .getWMessage('/syndata/tenantryHost/getTenantryHostFeature', {
        id: id || tenantryHostId,
      })
      .then((res) => setDataSource(res.data));
  }
  function handleEditCharacteristic(id, feature) {
    getCharacteristic(id).then(() => {
      setFeatureType(feature);
      setTenantryHostId(id);
      setCharacteristicTrue();
    });
  }

  function handleDeleteFeature(deleteId) {
    request
      .deleteWMessage('/syndata/tenantryHost/deleteTenantryHostFeature', { id: deleteId })
      .then(() => getCharacteristic());
  }
  function handleDelete(deleteId) {
    request
      .deleteWMessage('/syndata/tenantryHost/deleteTenantryHost', { id: deleteId })
      .then(() => refresh());
  }
  function handleAdd() {
    form.setFieldsValue({ tenantryName: tenantKeyMap[currentKey], tenantryId: currentKey });
    setTrue();
  }
  function handleFormSubmit() {
    return form.validateFields().then((values) => {
      request.postWMessage('/syndata/tenantryHost/saveTenantryHost', values).then(() => {
        handleCancel();
        refresh();
      });
    });
  }
  function handleCancel() {
    form.resetFields();
    setFalse();
    setFeatureType('');
    setTenantryHostId('');
  }
  function onTabChange(tabKey) {
    setCurrentKey(tabKey);
    run({ tenantryType: tabKey });
  }
  function onSave(_, values) {
    return request
      .postWMessage('/syndata/tenantryHost/saveTenantryHostFeature', {
        ...values,
        tenantryHostId,
      })
      .then(() => getCharacteristic());
  }
  function fetchTabList() {
    return request.getWMessage('/system/dict/option', { dictType: 'YHLX' }).then((res) => {
      const { data: values } = res;
      const resultData = values.map((i) => ({
        key: i.dictValue,
        tab: i.dictLabel,
      }));
      const keyMap = resultData.reduce((i, item) => {
        i[item.key] = item.tab;
        return i;
      }, {});
      setTabList(resultData);
      setTenantKeyMap(keyMap);
      run({ tenantryType: resultData[0].key });
      setCurrentKey(resultData[0].key);
    });
  }

  function onChange(page, filters) {
    const params = {
      current: page.current,
      pageSize: page.pageSize,
      ...filters,
    };
    run({ ...params, ...searchForm.getFieldsValue(), tenantryType: currentKey });
  }

  return (
    <Row gutter={12} className="fullBox">
      <Col className={styles.indexPageBox} span={24}>
        <Card
          title={<span style={{ fontWeight: 'bolder', fontSize: '18px' }}>主机群管理</span>}
          extra={
            <Button type="primary" onClick={handleAdd}>
              创建主机
            </Button>
          }
          className="!mb-4"
        >
          <SearchBar
            onSearch={(value) => {
              run({ pageSize: pagination.pageSize, ...value, tenantryType: currentKey });
            }}
            itemList={searchBarItem}
            form={searchForm}
          />
        </Card>

        <Card className={styles.tableListStyle} tabList={tabList} onTabChange={onTabChange}>
          <Table
            rowKey="id"
            size="middle"
            bordered
            columns={tableCol}
            dataSource={data.list}
            onChange={onChange}
            pagination={{
              ...pagination,
              showQuickJumper: true,
              showSizeChanger: true,
              showTotal: (total) => `共 ${total} 项`,
            }}
          />
        </Card>
        <Modal
          closeIcon
          title="主机信息维护"
          open={visible}
          onCancel={handleCancel}
          onOk={handleFormSubmit}
          centered
          width={700}
        >
          <Form form={form} {...formItemLayout}>
            <FormBuilder items={formItems} />
          </Form>
        </Modal>
        <Drawer
          open={characteristicVisible}
          closeIcon
          title="特征值配置"
          width="50%"
          footerStyle={{ textAlign: 'right' }}
          footer={<Button onClick={setCharacteristicFalse}>关闭</Button>}
          onClose={setCharacteristicFalse}
        >
          <Alert
            className="mb-4"
            message={
              <>
                当前主机所属租户为<span className="text-blue-6">{tenantKeyMap[currentKey]}</span>
                请配置{CharacteristicsTypeName[featureType]}作为主机特征值
              </>
            }
          />
          <EditableProTable
            rowKey="id"
            columns={characteristicColumns}
            value={dataSource}
            onChange={setDataSource}
            recordCreatorProps={{
              record: () => ({ id: uuid().replace(/-/g, '') }),
            }}
            editable={{
              type: 'multiple',
              editTableKeys,
              onSave,
              onChange: setEditTableKeys,
              onlyAddOneLineAlertMessage: '只能同时新增一行数据',
              actionRender: (row, config, defaultDom) => [defaultDom.save, defaultDom.cancel],
            }}
            cardProps={{ bodyStyle: { padding: 0 } }}
          />
        </Drawer>
      </Col>
    </Row>
  );
}
export default HostGroupManagement;
