import { Button, Card, Form, Modal, Popconfirm, Space, Switch, Table } from 'antd';
import request from '@/utils/request';
import { useBoolean } from 'ahooks';
import FormBuilder from '@/components/forms/FormBuilder';
import {
  dataModelEndCol,
  dataModelItem,
  dataModelStartCol,
} from '@/pages/system/modelManagement/ModelColumns';
import RemoteSelect from '@/components/forms/RemoteSelect';
import MetaDrawer from '@/components/system/modelManagement/MetaDrawer';
import { useState } from 'react';
import { ManagePage, withManagePage } from '@/components/hoc/ManagePage';
import DataModelFormItem from '@/components/system/modelManagement/DataModelFormItem';

function DataModel({ refreshTableData }) {
  const [visible, { setTrue, setFalse }] = useBoolean(false);
  const [metadata, setMetadata] = useState({});
  const [metadataVisible, { setTrue: setMetadataVisibleTrue, setFalse: setMetadataVisibleFalse }] =
    useBoolean(false);
  const [form] = Form.useForm();
  const formItemLayout = { labelCol: { span: 4 }, wrapperCol: { span: 18 } };

  const isUse = {
    key: 'isTableShard',
    dataIndex: 'isTableShard',
    title: '是否分表',
    component: Switch,
    width: 80,
    align: 'center',
    componentProps: {
      unCheckedChildren: '否',
      checkedChildren: '是',
    },
    formItemProps: {
      valuePropName: 'checked',
    },
    render: (text, record) => (
      <Switch
        checked={text}
        checkedChildren="是"
        unCheckedChildren="否"
        onChange={(state) => handleSave({ ...record, isTableShard: state })}
      />
    ),
  };
  const columns = [
    {
      title: '序号',
      dataIndex: 'index',
      key: 'index',
      width: 50,
      align: 'center',
      render: (text, record, index) => record?.pid === '0' && index + 1,
    },
    ...dataModelStartCol,
    isUse,
    ...dataModelEndCol,
    {
      title: '操作',
      width: 200,
      align: 'center',
      render: (_, record) => {
        return (
          <Space size={0}>
            <Button type="link" onClick={() => handleEdit(record)} size="small">
              编辑
            </Button>
            <Popconfirm
              title={
                record.pid === '0'
                  ? '删除主模型会将所属副模型一起删除，确认删除该数据模型吗？'
                  : '确认删除该数据模型吗？'
              }
              onConfirm={() => handleDelete(record.id)}
              size="small"
            >
              <Button type="link">删除</Button>
            </Popconfirm>
            <Button type="link" onClick={() => handleDrawerEdit(record)} size="small">
              元数据配置
            </Button>
            {record.pid === '0' && (
              <Button type="link" onClick={() => handleSecondAdd(record)} size="small">
                添加副模型
              </Button>
            )}
          </Space>
        );
      },
    },
  ];
  function handleDrawerEdit(record) {
    setMetadata({
      modelName: record.modelName,
      modelIdentification: record.modelIdentification,
      tableName: record.tableName,
      id: record.id,
      dbName: record.dbName,
    });
    setMetadataVisibleTrue();
  }
  function handleEdit(record) {
    setTrue();
    form.setFieldsValue({ ...record, modelType: record.pid === '0' ? '主模型' : '副模型' });
  }

  function handleCancel() {
    setFalse();
    form.resetFields();
  }

  function handleDelete(deleteId) {
    request
      .deleteWMessage('/syndata/dataModel/deleteDataModel', { id: deleteId })
      .then(refreshTableData);
  }

  function handleFormSubmit() {
    return form.validateFields().then((values) => {
      const params = { ...values, modelType: values.pid === '0' };
      handleSave(params);
    });
  }

  function handleSave(newTableData) {
    request.postWMessage('/syndata/dataModel/saveMainDataModel', newTableData).then(() => {
      refreshTableData();
      setFalse();
      form.resetFields();
    });
  }

  function handleAdd() {
    form.setFieldsValue({ pid: '0', modelType: '主模型' });
    setTrue();
  }

  function handleSecondAdd(record) {
    setTrue();
    form.setFieldsValue({
      modelType: '副模型',
      pid: record.id,
      businessModelId: record.businessModelId,
      businessModelName: record.businessModelName,
      dbName: record.dbName,
    });
  }

  return (
    <>
      <Card
        title="数据模型管理"
        extra={
          <Button onClick={handleAdd} type="primary">
            新增主模型
          </Button>
        }
      >
        <ManagePage tableColumns={columns} searchBarItemList={dataModelItem} />
      </Card>
      <Modal
        title={form.getFieldValue('pid') === '0' ? '数据主模型维护' : '数据副模型维护'}
        open={visible}
        onCancel={handleCancel}
        onOk={handleFormSubmit}
        centered
        width={700}
        closeIcon
      >
        <Form form={form} {...formItemLayout}>
          <FormBuilder items={DataModelFormItem({ form })} />
        </Form>
      </Modal>
      <MetaDrawer
        open={metadataVisible}
        setMetadataVisibleFalse={setMetadataVisibleFalse}
        record={metadata}
      />
    </>
  );
}
export default withManagePage(DataModel, '/syndata/dataModel/getDataModel');
