import { Button, Col, Drawer, Popconfirm, Row, Space, Table } from 'antd';
import styles from '@/pages/system/syncModel/SyncModel.module.less';
import RuleTable from '@/pages/system/syncModel/RuleTable';
import { FunctionType, FunctionTypeMap } from '@/pages/system/syncProcess/ProcessStatus';
import { useEffect, useState } from 'react';
import request from '@/utils/request';
import { EditableProTable } from '@ant-design/pro-table';
import { v4 as uuid } from 'uuid';
import TitleStyle from '@/layouts/header/TitleStyle';

function FilterRulesDrawer(props) {
  const { open, setFalse, dataModelId } = props;
  const [selectedRowKeys, setSelectRowKeys] = useState();
  const [selectedType, setSelectedType] = useState(false);
  const [characteristicTableKeys, setCharacteristicTableKeys] = useState('');
  const [characteristicDataSource, setCharacteristicDataSource] = useState([]);
  const [dataSource, setDataSource] = useState([]);
  const [editableKeys, setEditableKeys] = useState([]);
  const drawerItems = [
    { key: FunctionType.INCREMENT, name: FunctionTypeMap[FunctionType.INCREMENT] },
    { key: FunctionType.COMPLETE, name: FunctionTypeMap[FunctionType.COMPLETE] },
  ];
  const columns = [
    { title: '过滤规则名称', width: 180, dataIndex: 'filterRuleName', key: 'filterRuleName' },
    {
      title: '操作',
      align: 'center',
      width: 180,
      dataIndex: 'option',
      valueType: 'option',
      render: (text, record, _, action) => {
        if (!record.isGlobal) {
          return (
            <Space>
              <Button
                type="link"
                size="small"
                onClick={() => {
                  action?.startEditable?.(record.id);
                }}
              >
                编辑
              </Button>
              <Popconfirm title="确认删除吗?" onConfirm={() => handleDeleteFeature(record.id)}>
                <Button type="link" size="small">
                  删除
                </Button>
              </Popconfirm>
            </Space>
          );
        }
        return '-';
      },
    },
  ];
  const characteristicColumns = [
    { title: '特征值', align: 'center', width: 180, dataIndex: 'eigenvalue', key: 'eigenvalue' },
    { title: '说明', width: 180, dataIndex: 'comment', key: 'comment', align: 'center' },
    {
      title: '操作',
      align: 'center',
      width: 180,
      dataIndex: 'option',
      valueType: 'option',
      render: (text, record, _, action) => {
        return (
          <Space>
            <Button
              type="link"
              size="small"
              onClick={() => {
                action?.startEditable?.(record.id);
              }}
            >
              编辑
            </Button>
            <Popconfirm title="确认删除吗?" onConfirm={() => handleDeleteEigenvalue(record.id)}>
              <Button type="link" size="small">
                删除
              </Button>
            </Popconfirm>
          </Space>
        );
      },
    },
  ];
  useEffect(() => {
    if (dataModelId) {
      fetchTableData();
    }
  }, [open]);
  function handleCancel() {
    setSelectRowKeys(null);
    setFalse();
  }
  function fetchTableData() {
    return request
      .getWMessage('/syndata/syncModel/getSyncModelFilterNameList', { id: dataModelId })
      .then((res) => {
        const { data } = res;
        setDataSource(data || []);
        if (data?.length > 0) {
          setSelectRowKeys(data[0]?.id);
          setSelectedType(data[0]?.isGlobal);
          fetchCharacteristic(data[0].id);
        }
      });
  }
  function fetchCharacteristic(characteristicId) {
    return request
      .getWMessage('/syndata/syncModel/getSyncModelFilterFeature', {
        id: characteristicId,
      })
      .then((res) => {
        setCharacteristicDataSource(res.data || []);
      });
  }
  function setRowClassName(record) {
    return record.id === selectedRowKeys ? styles.rowStyle : '';
  }

  function onSelectChange(record) {
    setSelectRowKeys(record.id);
    setSelectedType(record.isGlobal);
    fetchCharacteristic(record.id);
  }
  function handleDeleteFeature(deleteId) {
    request
      .deleteWMessage('/syndata/syncModel/deleteSyncModelFilterName', { id: deleteId })
      .then(fetchTableData);
  }

  function handleDeleteEigenvalue(deleteId) {
    return request
      .deleteWMessage('/syndata/syncModel/deleteSyncModelFilterFeature', {
        id: deleteId,
      })
      .then(() => {
        fetchCharacteristic(selectedRowKeys);
      });
  }
  function handleSave(_, values) {
    return request
      .postWMessage('/syndata/syncModel/saveSyncModelFilterName', {
        ...values,
        dataId: dataModelId,
      })
      .then(() => {
        setSelectRowKeys(values.id);
        setSelectedType(values.isGlobal);
      });
  }
  function handleSaveCharacteristic(_, values) {
    return request.postWMessage('/syndata/syncModel/saveSyncModelFilterFeature', {
      ...values,
      filterConfigId: selectedRowKeys,
    });
  }
  return (
    <Drawer
      title="过滤规则"
      open={open}
      onClose={handleCancel}
      width={'65vw'}
      closeIcon
      footerStyle={{ textAlign: 'right' }}
      footer={<Button onClick={handleCancel}>关闭</Button>}
    >
      <Row className="h-full" gutter={24}>
        <Col span={8} className={styles.leftColBorder}>
          <EditableProTable
            value={dataSource}
            columns={columns}
            rowClassName={setRowClassName}
            className="px-2 pt-2"
            rowKey="id"
            pagination={false}
            onChange={setDataSource}
            size="small"
            editable={{
              type: 'multiple',
              editableKeys,
              onChange: setEditableKeys,
              onSave: handleSave,
              actionRender: (row, config, defaultDom) => [defaultDom.save, defaultDom.cancel],
            }}
            onRow={(record) => ({
              onClick: () => {
                if (record.filterRuleName && editableKeys?.length === 0) {
                  onSelectChange(record);
                }
              },
            })}
            recordCreatorProps={{
              creatorButtonText: '添加行',
              onClick: () => setSelectRowKeys(null),
              record: () => ({ id: uuid().replace(/-/g, ''), isGlobal: false }),
            }}
          />
        </Col>
        <Col span={16}>
          {selectedRowKeys && (
            <>
              {!selectedType && (
                <>
                  <TitleStyle className="text-base">特征值配置</TitleStyle>
                  <EditableProTable
                    rowKey="id"
                    columns={characteristicColumns}
                    value={characteristicDataSource}
                    onChange={setCharacteristicDataSource}
                    recordCreatorProps={{
                      record: () => ({ id: uuid().replace(/-/g, '') }),
                    }}
                    className="mt-2"
                    editable={{
                      type: 'multiple',
                      characteristicTableKeys,
                      onSave: handleSaveCharacteristic,
                      onChange: setCharacteristicTableKeys,
                      onlyAddOneLineAlertMessage: '只能同时新增一行数据',
                      creatorButtonText: '添加行',
                      actionRender: (row, config, defaultDom) => [
                        defaultDom.save,
                        defaultDom.cancel,
                      ],
                    }}
                    cardProps={{ bodyStyle: { padding: 0 } }}
                  />
                </>
              )}

              {drawerItems.map((i) => (
                <RuleTable
                  dataModelId={dataModelId}
                  drawerActiveKey={i.key}
                  key={i.key}
                  title={i.name}
                  filterConfigId={selectedRowKeys}
                />
              ))}
            </>
          )}
        </Col>
      </Row>
    </Drawer>
  );
}
export default FilterRulesDrawer;
