import { ManagePage, withManagePage } from '@/components/hoc/ManagePage';
import { Button, Card, Modal, Popconfirm, Select, Space, Tag } from 'antd';
import request from '@/utils/request';
import RemoteSelect from '@/components/forms/RemoteSelect';
import { TenantUserTypeColor } from '@/const/TenantType';
import { Link, useHistory } from 'react-router-dom';
import {
  SyncStatus,
  SyncStatusMap,
  syncStatusOption,
} from '@/pages/system/syncProcess/ProcessStatus';
import React from 'react';
import { ExclamationCircleFilled } from '@ant-design/icons';

const { confirm } = Modal;
function SyncTaskList({ refreshTableData }) {
  const history = useHistory();
  const columns = [
    {
      title: '序号',
      dataIndex: 'index',
      key: 'index',
      width: 50,
      align: 'center',
      render: (text, record, index) => index + 1,
    },
    {
      title: '模型名称',
      dataIndex: 'taskName',
      key: 'taskName',
      width: 180,
      align: 'center',
    },
    {
      title: '同步业务',
      dataIndex: 'modelName',
      key: 'modelName',
      width: 180,
      align: 'center',
    },
    {
      title: '同步通道',
      dataIndex: 'syncThoroughfare',
      key: 'syncThoroughfare',
      width: 80,
      align: 'center',
    },
    {
      dataIndex: 'tenantryUserType',
      key: 'tenantryUserType',
      title: '源租户',
      align: 'center',
      width: 120,
      render: (text, record) => (
        <Tag color={TenantUserTypeColor[text]} className="py-1 px-1 w-24 text-sm text-center">
          {record.tenantTypeName}
        </Tag>
      ),
    },
    {
      title: '目标租户',
      dataIndex: 'targetTenant',
      key: 'targetTenant',
      width: 100,
      align: 'center',
      render: (text) => <span className="text-blue-6">{text}</span>,
    },
    {
      title: '状态',
      dataIndex: 'status',
      key: 'status',
      width: 100,
      align: 'center',
      render: (text) => {
        if (text === SyncStatus.SUCCESS) {
          return <Tag color="green">{SyncStatusMap[text]}</Tag>;
        }
        if (text === SyncStatus.ERROR) {
          return <Tag color="red">{SyncStatusMap[text]}</Tag>;
        }
        return <Tag>{SyncStatusMap[SyncStatus.TO_COMPILED]}</Tag>;
      },
    },
    {
      title: '操作',
      width: 120,
      align: 'center',
      render: (_, record) => {
        return (
          <Space size="small">
            {record.status === SyncStatus.TO_COMPILED || record.status === null ? (
              <>
                <Button type="link" onClick={() => handleStart(record.id)} size="small">
                  启动
                </Button>
                <Button
                  type="link"
                  size="small"
                  onClick={() => history.push(`/system/syncProcessManage/${record.id}`)}
                >
                  任务配置
                </Button>
              </>
            ) : (
              <>
                <Button type="link" size="small" onClick={() => handleStop(record)}>
                  停止
                </Button>
                <Button
                  type="link"
                  size="small"
                  onClick={() => history.push(`/system/HostMaintenanceEdit/${record.id}`)}
                >
                  主机维护
                </Button>
              </>
            )}
            <Button
              type="link"
              size="small"
              onClick={() => history.push(`/system/syncProcessManage/details/${record.id}`)}
            >
              监听详情
            </Button>
            <Popconfirm title="确定删除吗？" onConfirm={() => handleDelete(record.id)}>
              <Button type="link">删除</Button>
            </Popconfirm>
          </Space>
        );
      },
    },
  ];
  const searchBarItemList = [
    {
      name: 'keyword',
      key: 'keyword',
      label: '模糊搜索',
      componentProps: { placeholder: '任务名称', style: { width: 180 } },
    },
    {
      name: 'businessModeId',
      key: 'businessModeId',
      label: '同步业务',
      component: RemoteSelect,
      componentProps: {
        requestUrl: '/syndata/dataModel/getBusinessModel',
        titleAttr: 'modelName',
        valueAttr: 'id',
        style: { minWidth: 180 },
      },
    },
    {
      name: 'tenantryUserType',
      key: 'tenantryUserType',
      label: '源租户',
      component: RemoteSelect,
      componentProps: {
        allowClear: true,
        placeholder: '请选择',
        requestUrl: '/system/dict/option',
        requestParam: { dictType: 'YHLX' },
        titleAttr: 'dictLabel',
        valueAttr: 'dictValue',
        style: { minWidth: 180 },
      },
    },
    {
      key: 'status',
      label: '状态',
      component: Select,
      componentProps: {
        style: { width: 180 },
        options: syncStatusOption,
      },
    },
  ];
  function handleDelete(deleteId) {
    request
      .deleteWMessage('/syndata/syncTask/deleteSyncTask', { id: deleteId })
      .then(() => refreshTableData());
  }
  function handleStop(record) {
    confirm({
      title: '停止同步任务后重新启动将清除全部结构及数据!',
      icon: <ExclamationCircleFilled />,
      okText: '确定',
      cancelText: '取消',
      centered: true,
      onOk() {
        return request
          .postWMessage('/syndata/syncTask/pauseSyncTask', {
            id: record.id,
          })
          .then(() => refreshTableData());
      },
    });
  }

  function handleStart(id) {
    confirm({
      title: '启动将同步全部主机结构、全量数据以及增量数据!',
      icon: <ExclamationCircleFilled />,
      okText: '确定',
      cancelText: '取消',
      centered: true,
      onOk() {
        return request
          .postWMessage('/syndata/syncTask/startOrPauseSyncTask', {
            id,
          })
          .then(() => {
            refreshTableData();
          });
      },
    });
  }

  return (
    <Card
      title="同步任务管理"
      extra={
        <Link to="/system/syncProcessManage">
          <Button type="primary">创建任务</Button>
        </Link>
      }
    >
      <ManagePage tableColumns={columns} searchBarItemList={searchBarItemList} />
    </Card>
  );
}

export default withManagePage(SyncTaskList, '/syndata/syncTask/getSyncTask');
